@@ -44,11 +44,12 @@ PairEndProcessor::~PairEndProcessor() {
 }
 
 void PairEndProcessor::initOutput() {
-    if(mOptions->out1.empty() || mOptions->out2.empty())
+    if(mOptions->out1.empty())
         return;
     
     mLeftWriter = new WriterThread(mOptions, mOptions->out1);
-    mRightWriter = new WriterThread(mOptions, mOptions->out2);
+    if(!mOptions->out2.empty())
+        mRightWriter = new WriterThread(mOptions, mOptions->out2);
 }
 
 void PairEndProcessor::closeOutput() {
@@ -140,9 +141,11 @@ bool PairEndProcessor::process(){
     cerr << endl;
     cerr << "Read2 before filtering:"<<endl;
     finalPreStats2->print();
-    cerr << endl;
-    cerr << "Read2 aftering filtering:"<<endl;
-    finalPostStats2->print();
+    if(!mOptions->merge.enabled) {
+        cerr << endl;
+        cerr << "Read2 aftering filtering:"<<endl;
+        finalPostStats2->print();
+    }
 
     cerr << endl;
     cerr << "Filtering result:"<<endl;
@@ -167,6 +170,18 @@ bool PairEndProcessor::process(){
     cerr << endl;
     cerr << "Insert size peak (evaluated by paired-end reads): " << peakInsertSize << endl;
 
+    if(mOptions->merge.enabled) {
+        cerr << endl;
+        cerr << "Read pairs merged: " << finalFilterResult->mMergedPairs << endl;
+        if(finalPostStats1->getReads() > 0) {
+            double postMergedPercent = 100.0 * finalFilterResult->mMergedPairs / finalPostStats1->getReads();
+            double preMergedPercent = 100.0 * finalFilterResult->mMergedPairs / finalPreStats1->getReads();
+            cerr << "% of original read pairs: " << preMergedPercent << "%" << endl;
+            cerr << "% in reads after filtering: " << postMergedPercent << "%" << endl;
+        }
+        cerr << endl;
+    }
+
     // make JSON report
     JsonReporter jr(mOptions);
     jr.setDupHist(dupHist, dupMeanGC, dupRate);
@@ -227,8 +242,9 @@ int PairEndProcessor::getPeakInsertSize() {
 bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
     string outstr1;
     string outstr2;
-    string interleaved;
+    string singleOutput;
     int readPassed = 0;
+    int mergedCount = 0;
     for(int p=0;p<pack->count;p++){
         ReadPair* pair = pack->data[p];
         Read* or1 = pair->mLeft;
@@ -258,28 +274,28 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
             mUmiProcessor->process(or1, or2);
 
         // trim in head and tail, and apply quality cut in sliding window
-        Read* r1 = mFilter->trimAndCut(or1, mOptions->trim.front1, mOptions->trim.tail1);
-        Read* r2 = mFilter->trimAndCut(or2, mOptions->trim.front2, mOptions->trim.tail2);
+        int frontTrimmed1 = 0;
+        int frontTrimmed2 = 0;
+        Read* r1 = mFilter->trimAndCut(or1, mOptions->trim.front1, mOptions->trim.tail1, frontTrimmed1);
+        Read* r2 = mFilter->trimAndCut(or2, mOptions->trim.front2, mOptions->trim.tail2, frontTrimmed2);
 
         if(r1 != NULL && r2!=NULL) {
             if(mOptions->polyGTrim.enabled)
                 PolyX::trimPolyG(r1, r2, config->getFilterResult(), mOptions->polyGTrim.minLen);
-            if(mOptions->polyXTrim.enabled)
-                PolyX::trimPolyX(r1, r2, config->getFilterResult(), mOptions->polyXTrim.minLen);
         }
         bool isizeEvaluated = false;
         if(r1 != NULL && r2!=NULL && (mOptions->adapter.enabled || mOptions->correction.enabled)){
             OverlapResult ov = OverlapAnalysis::analyze(r1, r2, mOptions->overlapDiffLimit, mOptions->overlapRequire);
             // we only use thread 0 to evaluae ISIZE
             if(config->getThreadId() == 0) {
-                statInsertSize(r1, r2, ov);
+                statInsertSize(r1, r2, ov, frontTrimmed1, frontTrimmed2);
                 isizeEvaluated = true;
             }
             if(mOptions->correction.enabled) {
                 BaseCorrector::correctByOverlapAnalysis(r1, r2, config->getFilterResult(), ov);
             }
             if(mOptions->adapter.enabled) {
-                bool trimmed = AdapterTrimmer::trimByOverlapAnalysis(r1, r2, config->getFilterResult(), ov);
+                bool trimmed = AdapterTrimmer::trimByOverlapAnalysis(r1, r2, config->getFilterResult(), ov, frontTrimmed1, frontTrimmed2);
                 if(!trimmed){
                     if(mOptions->adapter.hasSeqR1)
                         AdapterTrimmer::trimBySequence(r1, config->getFilterResult(), mOptions->adapter.sequence, false);
@@ -291,10 +307,15 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
 
         if(config->getThreadId() == 0 && !isizeEvaluated && r1 != NULL && r2!=NULL) {
             OverlapResult ov = OverlapAnalysis::analyze(r1, r2, mOptions->overlapDiffLimit, mOptions->overlapRequire);
-            statInsertSize(r1, r2, ov);
+            statInsertSize(r1, r2, ov, frontTrimmed1, frontTrimmed2);
             isizeEvaluated = true;
         }
 
+        if(r1 != NULL && r2!=NULL) {
+            if(mOptions->polyXTrim.enabled)
+                PolyX::trimPolyX(r1, r2, config->getFilterResult(), mOptions->polyXTrim.minLen);
+        }
+
         if(r1 != NULL && r2!=NULL) {
             if( mOptions->trim.maxLen1 > 0 && mOptions->trim.maxLen1 < r1->length())
                 r1->resize(mOptions->trim.maxLen1);
@@ -302,25 +323,60 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
                 r2->resize(mOptions->trim.maxLen2);
         }
 
-        int result1 = mFilter->passFilter(r1);
-        int result2 = mFilter->passFilter(r2);
+        Read* merged = NULL;
+        // merging mode
+        if(mOptions->merge.enabled && r1 && r2) {
+            OverlapResult ov = OverlapAnalysis::analyze(r1, r2, mOptions->overlapDiffLimit, mOptions->overlapRequire);
+            if(ov.overlapped) {
+                merged = OverlapAnalysis::merge(r1, r2, ov);
+                int result = mFilter->passFilter(merged);
+                config->addFilterResult(result, 2);
+                if(result == PASS_FILTER) {
+                    singleOutput += merged->toString();
+                    config->getPostStats1()->statRead(merged);
+                    readPassed++;
+                    mergedCount++;
+                }
+                delete merged;
+            } else if(!mOptions->merge.discardUnmerged){
+                int result1 = mFilter->passFilter(r1);
+                config->addFilterResult(result1, 1);
+                if(result1 == PASS_FILTER) {
+                    singleOutput += r1->toString();
+                    config->getPostStats1()->statRead(r1);
+                }
 
-        config->addFilterResult(max(result1, result2));
-
-        if( r1 != NULL &&  result1 == PASS_FILTER && r2 != NULL && result2 == PASS_FILTER ) {
-            
-            if(mOptions->outputToSTDOUT) {
-                interleaved += r1->toString() + r2->toString();
-            } else {
-                outstr1 += r1->toString();
-                outstr2 += r2->toString();
+                int result2 = mFilter->passFilter(r2);
+                config->addFilterResult(result2, 1);
+                if(result2 == PASS_FILTER) {
+                    singleOutput += r2->toString();
+                    config->getPostStats1()->statRead(r2);
+                }
+                if(result1 == PASS_FILTER && result2 == PASS_FILTER )
+                    readPassed++;
             }
+        } else {
 
-            // stats the read after filtering
-            config->getPostStats1()->statRead(r1);
-            config->getPostStats2()->statRead(r2);
+            int result1 = mFilter->passFilter(r1);
+            int result2 = mFilter->passFilter(r2);
 
-            readPassed++;
+            config->addFilterResult(max(result1, result2), 2);
+
+            if( r1 != NULL &&  result1 == PASS_FILTER && r2 != NULL && result2 == PASS_FILTER ) {
+                
+                if(mOptions->outputToSTDOUT) {
+                    singleOutput += r1->toString() + r2->toString();
+                } else {
+                    outstr1 += r1->toString();
+                    outstr2 += r2->toString();
+                }
+
+                // stats the read after filtering
+                config->getPostStats1()->statRead(r1);
+                config->getPostStats2()->statRead(r2);
+
+                readPassed++;
+            }
         }
 
         delete pair;
@@ -336,7 +392,7 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
         mOutputMtx.lock();
     if(mOptions->outputToSTDOUT) {
         // STDOUT output
-        fwrite(interleaved.c_str(), 1, interleaved.length(), stdout);
+        fwrite(singleOutput.c_str(), 1, singleOutput.length(), stdout);
     } else if(mOptions->split.enabled) {
         // split output by each worker thread
         if(!mOptions->out1.empty())
@@ -355,10 +411,10 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
             memcpy(rdata, outstr2.c_str(), outstr2.size());
             mRightWriter->input(rdata, outstr2.size());
         } else if(mLeftWriter) {
-            // write interleaved
-            char* ldata = new char[interleaved.size()];
-            memcpy(ldata, interleaved.c_str(), interleaved.size());
-            mLeftWriter->input(ldata, interleaved.size());
+            // write singleOutput
+            char* ldata = new char[singleOutput.size()];
+            memcpy(ldata, singleOutput.c_str(), singleOutput.size());
+            mLeftWriter->input(ldata, singleOutput.size());
         }
     }
     if(!mOptions->split.enabled)
@@ -369,19 +425,23 @@ bool PairEndProcessor::processPairEnd(ReadPairPack* pack, ThreadConfig* config){
     else
         config->markProcessed(pack->count);
 
+    if(mOptions->merge.enabled) {
+        config->addMergedPairs(mergedCount);
+    }
+
     delete pack->data;
     delete pack;
 
     return true;
 }
     
-void PairEndProcessor::statInsertSize(Read* r1, Read* r2, OverlapResult& ov) {
+void PairEndProcessor::statInsertSize(Read* r1, Read* r2, OverlapResult& ov, int frontTrimmed1, int frontTrimmed2) {
     int isize = mOptions->insertSizeMax;
     if(ov.overlapped) {
         if(ov.offset > 0)
-            isize = r1->length() + r2->length() - ov.overlap_len;
+            isize = r1->length() + r2->length() - ov.overlap_len + frontTrimmed1 + frontTrimmed2;
         else
-            isize = ov.overlap_len;
+            isize = ov.overlap_len + frontTrimmed1 + frontTrimmed2;
     }
 
     if(isize > mOptions->insertSizeMax)
